import ch.nevis.esauth.auth.engine.AuthResponse
import groovy.xml.XmlSlurper
import groovy.json.JsonSlurper

def cleanSession() {
  def s = request.getAuthSession(true)

  s.removeAttribute('agov.ident.rpcode.backup')
  s.removeAttribute('agov.ident.rpcode')
  s.removeAttribute('agov.ident.entityId')
  s.removeAttribute('saml.response.statusCode')
  if (response.getActualRoles().length > 0) {
    def actualRoles = Arrays.copyOf(response.getActualRoles(), response.getActualRoles().length)
    actualRoles.each{ role -> response.removeActualRole(role) }
  }
}

// for auditing
def sourceIp = request.getLoginContext()['connection.HttpHeader.X-Real-IP'] ?: 'unknown'
def userAgent = request.getLoginContext()['connection.HttpHeader.user-agent'] ?: request.getLoginContext()['connection.HttpHeader.User-Agent'] ?: 'unknown'
def referer = request.getLoginContext()['connection.HttpHeader.referer'] ?: request.getLoginContext()['connection.HttpHeader' +
		'.Referer'] ?: '-'
def origin = request.getLoginContext()['connection.HttpHeader.origin'] ?: request.getLoginContext()['connection.HttpHeader' +
		'.Origin'] ?: '-'

// 1) we need to know the code of the RP
def rpcode = inargs['rpcode'] ?: inargs['RelayState']
def rpcodeBackup = session['agov.ident.rpcode']

if (rpcode)
{
  if (rpcodeBackup) {
    response.setSessionAttribute('agov.ident.rpcode.backup', rpcodeBackup)
  }
  response.setSessionAttribute('agov.ident.rpcode', rpcode)
} else {
  cleanSession()
  LOG.info("Event='IDENT-INVALIDREQ', rpcode='missing', SourceIp=${sourceIp}, UserAgent=${userAgent}, Referer='${referer}', Origin='${origin}'")
  response.setResult('inavlidurl')
  return
}

// 2) load rp settings in session (if needed)
if (rpcode != rpcodeBackup) {
  def slurper = new JsonSlurper()
  def rpMap = slurper.parseText(parameters['rpcode.list'])
  LOG.debug(">>> rpMaP: ${rpMap}")
  if (!rpMap[rpcode]) {
    cleanSession()
    LOG.info("Event='IDENT-INVALIDREQ', rpcode='${rpcode}', SourceIp=${sourceIp}, UserAgent=${userAgent}, Referer='${referer}', Origin='${origin}'")
    response.setResult('inavlidurl')
    return
  }
  response.setSessionAttribute('agov.ident.entityId', rpMap[rpcode])
}

// 3) if we have a response ...
if (inargs['SAMLResponse']) {
    response.setResult('processResponse')
    return
}

// 4) otherwise
response.setResult('sendAuthnRequest')
return
